"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CacheError_1 = __importDefault(require("../errors/CacheError"));
const MemoryLeakWarning_1 = __importDefault(require("../errors/MemoryLeakWarning"));
const Pair_1 = __importDefault(require("./Pair"));
class Cache {
    constructor(options) {
        this.cache = new Map();
        this.options = {
            maxSize: null,
            maxAge: null,
            checkInterval: 60,
            removeOldest: true,
        };
        this.options = options !== null && options !== void 0 ? options : this.options;
        setInterval(() => {
            for (let [key, value] of this.cache.entries()) {
                if (this.options.maxAge && Date.now() - value.timestamp > this.options.maxAge) {
                    this.remove(key);
                }
            }
        }, this.options.checkInterval * 1000);
    }
    /**
     * Get a value from the cache. If the key is not found
     * in the cache, `null` is returned.
     *
     * @param {string} key The key to get the value for.
     * @returns {any | null} The value for the key, or `null` if the key is not found.
     * @public
     */
    get(key) {
        let entry = this.cache.get(key);
        return entry ? entry.v : null;
    }
    /**
     * Set a value in the cache.
     *
     * @param {string} key The key of the entry.
     * @param {any} value The value to set it to.
     * @returns {Cache} The updated cache.
     * @public
     */
    set(key, value, options) {
        let push = {
            t: Date.now(),
            v: value,
        };
        this.makeSpace(options);
        this.cache.set(key, push);
        return this;
    }
    /**
     * Check if the cache contains a key.
     *
     * @param {string} key The key of the entry.
     * @returns {boolean} `true` if the key is found, `false` otherwise.
     * @public
     */
    has(key) {
        return this.cache.has(key);
    }
    /**
     * Remove a key from the cache.
     *
     * @param {string} key The key of the entry.
     * @returns {Cache} The updated cache.
     * @public
     */
    remove(key) {
        this.cache.delete(key);
        return this;
    }
    /**
     * Get an object of the entries in the cache. The
     * key is the key of the entry, and the value is
     * the value of the entry.
     *
     * @returns {{ [key: string]: any }} The array.
     * @public
     */
    entries() {
        let entries = {};
        for (let [key, value] of this.cache.entries()) {
            entries[key] = value.v;
        }
        return entries;
    }
    /**
     * Get an array of the entries in the cache.
     *
     * @returns {Pair<string, any>[]} The array.
     * @public
     */
    array() {
        let entries = [];
        for (let [key, value] of this.cache.entries()) {
            entries.push(new Pair_1.default(key, value.v));
        }
        return entries;
    }
    /**
     * Clear the cache and its contents.
     *
     * @returns {Cache} The updated cache.
     * @public
     */
    clear() {
        this.cache.clear();
        return this;
    }
    /**
     * Make space in the cache for a new entry. You
     * **probably shouldn't** run this method directly
     * unless you know **EXACTLY** what you're doing.
     *
     * @returns {void}
     * @public
     */
    makeSpace(options) {
        var _a, _b, _c;
        options = options !== null && options !== void 0 ? options : {};
        options.maxAge = options.maxAge || this.options.maxAge || null;
        options.removeOldest = (_b = (_a = options.removeOldest) !== null && _a !== void 0 ? _a : this.options.removeOldest) !== null && _b !== void 0 ? _b : true;
        options.allowOverflow = (_c = options.allowOverflow) !== null && _c !== void 0 ? _c : false;
        if (this.options.maxSize && this.cache.size >= this.options.maxSize) {
            if (options.allowOverflow) {
                process.emitWarning(new MemoryLeakWarning_1.default('possible memory leak detected: overflow for cache has been permitted by options provided'));
                this.options.maxSize++;
            }
            else if (options.removeOldest) {
                let oldest = this.cache.entries().next().value;
                this.remove(oldest[0]);
            }
            else {
                throw new CacheError_1.default('cannot write to cache; cache is full');
            }
        }
    }
}
exports.default = Cache;
